#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define TDF_IDENTIFIER 0X8BA72A31UL

#define USED 1
#define UNUSED 0

#define FORWARD 1
#define BACKWARD 0


struct system_matrix  /* Emission tomographic data format */
{
  int type;             /* =1 => forward projection                       */
                        /*       i is the pixel index and j is proj index */
                        /* =0 => back projection                          */
                        /*       j is the pixel index and i is proj index */

  /* Note: the following comments assume a forward projection             */
  int Npix;             /* Number of pixels                               */
  int Mpro;             /* Number of projections                          */
  int n_x, n_y;         /* number of samples in x and y image coordinates */
                        /* Sparse matrix column is computed by:           */
                        /* j = m*n_x+n where m  and n are respectively    */
                        /* the y and x indices of the pixels location.    */
  int *n_row;           /* n_row[j] specifies the number of nonzero       */
                        /* elements in the j^th column of the sparse      */
                        /* matrix. Must be of length N.                   */

  float **val;          /* val[j][i] is the i^th nonzero element of the   */
                        /* j^th column of the projection matrix.          */ 
  int   **row;          /* row[j][i] is the true row associated with      */
                        /* sparse matrix element A[j][i].                 */

  struct image_geometry
  {
    int used;             /* 1 if this structure is used; 0 otherwise */
    double delta_x,x_0;   /* x = (i-x_0)*delta_x */
    double delta_y,y_0;   /* y = (i-y_0)*delta_y */
  } img_geom;

};



void tdf_init_xform(struct system_matrix *xform,
  int type,           /* =1 => forward projection                       */
                      /* =0 => back projection                          */
  int n_x,            /* number of samples in x image coordinate        */
  int n_y,            /* number of samples in y image coordinate        */
                      /* Sparse matrix column is computed by:           */
                      /* j = m*n_x+n where m  and n are respectively    */
                      /* the y and x indices of the pixels location.    */
  int Mpro,           /* number of projections                          */
  int NN              /* NN specifies the total number of nonzero       */
                      /* elements in the sparse matrix.                 */
  );

void tdf_set_img_geom(struct system_matrix *xform,
  int    used,          /* 1 => used; 0 => not used */
  double delta_x,       /* x = (i-x_0)*delta_x */
  double x_0,
  double delta_y,       /* y = (i-y_0)*delta_y */
  double y_0
  );


int tdf_write_system_matrix(struct system_matrix *xform, 
  char *fname     /* character string containing file name */
  );


int tdf_read_system_matrix(struct system_matrix *xform, 
  char *fname     /* character string containing file name */
  );

void destroy_system_matix(struct system_matrix *xform);


