/*
*       write_bouman.c
*       Interface between Fessler and Bouman's sparse matrix structures 
*       By C. Bouman and J. Fessler, Jan. 1995  
*/
#include "tdf.h"

static void *tdf_get_spc(int num, size_t size);


/*
*       write_bouman()
*       sublist_offset[i] gives the offset into the all_* arrays at which the
*       nonzero elements of the i'th column are stored.
*       sublist_length[i] gives how many nonzero elements in i'th column.
*/
int write_bouman(
char    *fname,                 /* name of output file                  */
float   *all_values,            /* [nwt] nonzero matrix values          */
unsigned short  *all_indeces,   /* [nwt] nonzero row indeces            */
int     nx,                     /* image size - horizontal              */
int     ny,                     /* image size - vertical                */
unsigned int *sublist_length,   /* [nx*ny] see above                    */
int     nb,                     /* # of sinogram 'bins' (radial samples) */
int     na,                     /* # of sinogram angles         */
double  pixel_size,             /* arbitrary units              */
double  x_offset,
double  y_offset)
{
        struct system_matrix xform;
        int NN,i;

        xform.type = FORWARD;
        xform.Npix = nx*ny;
        xform.Mpro = nb*na;
        xform.n_x = nx;
        xform.n_y = ny;

        xform.n_row = (int *)tdf_get_spc(xform.Npix, sizeof(int));
        NN = 0;
        for(i=0; i<xform.Npix; i++ ) {
          NN += sublist_length[i];
          xform.n_row[i] = sublist_length[i];
        }

        xform.val = (float **)tdf_get_spc(xform.Npix, sizeof(float *) );
        xform.val[0] = all_values;

        xform.row = (int **)tdf_get_spc(xform.Npix, sizeof(int *) );
        xform.row[0] = (int *)tdf_get_spc(NN, sizeof(int) );
        for(i=0; i<NN; i++) xform.row[0][i] = all_indeces[i];

        for(i=0; i<xform.Npix-1; i++ ) {
          xform.val[i+1] = xform.val[i] + sublist_length[i];
          xform.row[i+1] = xform.row[i] + sublist_length[i];
        }

        tdf_set_img_geom(&xform, USED, pixel_size, (nx-1)/2 + x_offset, 
          pixel_size, (ny-1)/2 + y_offset );

        return tdf_write_system_matrix(&xform, fname);
}


static void *tdf_get_spc(int num, size_t size)
{
        void *pt;

        if( (pt=calloc((size_t)num,size)) == NULL ) {
                fprintf(stderr, "\nERROR: tdf_get_spc ==> calloc() error\n");
                exit(-1);
        }
        return pt;
}
